@@ -59,7 +59,7 @@ class Mean(UnaryOperator, BaseMetric):
     def calc(self, dataframe: pd.DataFrame) -> float:
         data = self.get_operand_column(dataframe)
         numeric_data = pd.to_numeric(data, errors="coerce")
-        return numeric_data.mean()
+        return cast(float, numeric_data.mean())
 
 
 @dataclass
@@ -89,16 +89,18 @@ class Max(UnaryOperator, BaseMetric):
 
 @dataclass
 class Cardinality(UnaryOperator, BaseMetric):
-    def calc(self, dataframe: pd.DataFrame) -> int:
+    def calc(self, dataframe: pd.DataFrame) -> float:
         data = self.get_operand_column(dataframe)
-        return data.nunique()
+        if data.dtype.kind == "f":
+            return float("nan")
+        return cast(float, data.nunique())
 
 
 @dataclass
 class PercentEmpty(UnaryOperator, BaseMetric):
     def calc(self, dataframe: pd.DataFrame) -> float:
         data = self.get_operand_column(dataframe)
-        return data.isna().mean() * 100
+        return cast(float, data.isna().mean() * 100)
 
 
 @dataclass
