@@ -3,9 +3,9 @@ from typing import List, Optional
 import strawberry
 from strawberry.types import Info
 
-from phoenix.core import Dimension as CoreDimension
-from phoenix.server.api.context import Context
+from phoenix.core.model_schema import REFERENCE, ScalarDimension
 
+from ..context import Context
 from ..input_types.Granularity import Granularity
 from ..input_types.TimeRange import TimeRange
 from .DataQualityMetric import DataQualityMetric
@@ -16,6 +16,7 @@ from .ScalarDriftMetricEnum import ScalarDriftMetric
 from .TimeSeries import (
     DataQualityTimeSeries,
     DriftTimeSeries,
+    ensure_timeseries_parameters,
     get_data_quality_timeseries_data,
     get_drift_timeseries_data,
 )
@@ -27,10 +28,10 @@ class Dimension(Node):
     type: DimensionType = strawberry.field(
         description="Whether the dimension represents a feature, tag, prediction, or actual."
     )
-
     dataType: DimensionDataType = strawberry.field(
         description="The data type of the column. Categorical or numeric."
     )
+    dimension: strawberry.Private[ScalarDimension]
 
     @strawberry.field
     def drift_metric(
@@ -46,19 +47,12 @@ class Dimension(Node):
         exists, if no primary data exists in the input time range, or if the
         input time range is invalid.
         """
-        if info.context.model.reference_dataset is None:
+        model = info.context.model
+        if model[REFERENCE].empty:
             return None
-        if len(
-            data := get_drift_timeseries_data(
-                self.name,
-                info.context.model,
-                metric,
-                time_range,
-                dtype=self.dataType,
-            )
-        ):
-            return data.pop().value
-        return None
+        time_range, granularity = ensure_timeseries_parameters(model, time_range)
+        data = get_drift_timeseries_data(self.dimension, metric, time_range, granularity)
+        return data[0].value if len(data) else None
 
     @strawberry.field
     async def data_quality_metric(
@@ -67,16 +61,9 @@ class Dimension(Node):
         metric: DataQualityMetric,
         time_range: Optional[TimeRange] = None,
     ) -> Optional[float]:
-        if len(
-            data := get_data_quality_timeseries_data(
-                self.name,
-                info.context.model,
-                metric,
-                time_range,
-            )
-        ):
-            return data.pop().value
-        return None
+        time_range, granularity = ensure_timeseries_parameters(info.context.model, time_range)
+        data = get_data_quality_timeseries_data(self.dimension, metric, time_range, granularity)
+        return data[0].value if len(data) else None
 
     @strawberry.field(
         description=(
@@ -85,11 +72,8 @@ class Dimension(Node):
             " Missing values are excluded. Non-categorical dimensions return an empty list."
         )
     )  # type: ignore  # https://github.com/strawberry-graphql/strawberry/issues/1929
-    def categories(self, info: Info[Context, None]) -> List[str]:
-        for dim in info.context.model.dimensions:
-            if dim.name == self.name:
-                return dim.categories
-        return []
+    def categories(self) -> List[str]:
+        return list(self.dimension.categories)
 
     @strawberry.field(
         description=(
@@ -106,14 +90,11 @@ class Dimension(Node):
         time_range: TimeRange,
         granularity: Granularity,
     ) -> DataQualityTimeSeries:
+        time_range, granularity = ensure_timeseries_parameters(
+            info.context.model, time_range, granularity
+        )
         return DataQualityTimeSeries(
-            data=get_data_quality_timeseries_data(
-                self.name,
-                info.context.model,
-                metric,
-                time_range,
-                granularity,
-            )
+            data=get_data_quality_timeseries_data(self.dimension, metric, time_range, granularity)
         )
 
     @strawberry.field(
@@ -131,27 +112,23 @@ class Dimension(Node):
         time_range: TimeRange,
         granularity: Granularity,
     ) -> DriftTimeSeries:
-        if info.context.model.reference_dataset is None:
+        model = info.context.model
+        if model[REFERENCE].empty:
             return DriftTimeSeries(data=[])
+        time_range, granularity = ensure_timeseries_parameters(model, time_range, granularity)
         return DriftTimeSeries(
-            data=get_drift_timeseries_data(
-                self.name,
-                info.context.model,
-                metric,
-                time_range,
-                granularity,
-                dtype=self.dataType,
-            )
+            data=get_drift_timeseries_data(self.dimension, metric, time_range, granularity)
         )
 
 
-def to_gql_dimension(id_attr: int, dimension: CoreDimension) -> Dimension:
+def to_gql_dimension(id_attr: int, dimension: ScalarDimension) -> Dimension:
     """
     Converts a phoenix.core.Dimension to a phoenix.server.api.types.Dimension
     """
     return Dimension(
         id_attr=id_attr,
         name=dimension.name,
-        type=DimensionType[dimension.type.value],
-        dataType=DimensionDataType[dimension.data_type.value],
+        type=DimensionType.from_(dimension),
+        dataType=DimensionDataType.from_(dimension),
+        dimension=dimension,
     )
